// 秒寻科技
// 预测坐标模块
// zt
// 2025-02-24

#include "../../includes/includes.h"
#include "./lct_public.h"

// 导入全局变量
extern PUB_SYS_STATUS_T gg_stSysStatus;
extern LCT_COORDSEL_MEAN_COORD_T gLct_stMeanLctCoordList;
extern LCT_COORDSEL_ORIGINAL_COORD_T gLct_stOriginalLctCoordList;
extern int32 gLct_dwSpeedX;
extern int32 gLct_dwSpeedY;
extern LCT_FIRSTPATH_PREDICTION_DIRECTION_DATA_T gLct_astPredictionDirectionDataList[PUB_SYS_CARRIER_NUM];

// 根据历史坐标，预测当前一秒的坐标
// pstPredictionCoord:预测坐标（返回参数）
// pbyPredictionFlag:预测标识（返回参数），0：预测失败，1：预测成功
// pstAheadPointCoord:前一个点坐标（返回参数）
// dwBulidId:大楼编号
// dwLayerId:楼层编号
void lct_prediction_current_coord_by_history(int32 dwBulidId, int32 dwLayerId, COORD_T *pstPredictionCoord, uint8 *pbyPredictionFlag, COORD_T *pstAheadPointCoord)
{
    LCT_COORDSEL_MEAN_COORD_T *pstData = (LCT_COORDSEL_MEAN_COORD_T *)&gLct_stMeanLctCoordList; // 平滑定位点队列
    uint8 byMeanListLen = pstData->byListLen;                                                   // 平滑定位点队列长度

    LCT_COORDSEL_ORIGINAL_COORD_T *pstOriginalLctCoordList = &gLct_stOriginalLctCoordList; // 原始定位点队列
    uint8 byOriginalListLen = pstOriginalLctCoordList->byListLen;                          // 原始定位点队列长度

    COORD_T stPrediction = {0};

    // 处于同一大楼和楼层
    if (dwBulidId == gg_stSysStatus.dwBuildId && dwLayerId == gg_stSysStatus.dwLayerId)
    {
        // 系统处于重定位阶段
        if (gg_stSysStatus.byReLocationFlag)
        {
            memset(&gLct_stOriginalLctCoordList, 0, sizeof(LCT_COORDSEL_ORIGINAL_COORD_T)); // 原始坐标队列复位
            memset(&gLct_stMeanLctCoordList, 0, sizeof(LCT_COORDSEL_MEAN_COORD_T));         // 平滑坐标队列复位
            gg_stSysStatus.byBtsCaptureFlag = 0;                                            // 系统进入基站捕获状态

            // 预测失败
            *pbyPredictionFlag = 0;
            return;
        }

        // 平滑队列中的定位点不够
        if (byMeanListLen < LCT_COORDSEL_MEAN_COORD_LIST_LEN)
        {
            // 预测失败
            *pbyPredictionFlag = 0;
            return;
        }

        // 计算基础速度
        // 最近两次位移的加权平均（3:1权重）
        int32 dx = (3 * (pstData->astCoordList[3].dwX - pstData->astCoordList[2].dwX) + (pstData->astCoordList[2].dwX - pstData->astCoordList[1].dwX)) / 4;
        int32 dy = (3 * (pstData->astCoordList[3].dwY - pstData->astCoordList[2].dwY) + (pstData->astCoordList[2].dwY - pstData->astCoordList[1].dwY)) / 4;

        // 计算加速度
        // 计算二阶差分（加速度）
        int32 ddx = (pstData->astCoordList[3].dwX - 2 * pstData->astCoordList[2].dwX + pstData->astCoordList[1].dwX);
        int32 ddy = (pstData->astCoordList[3].dwY - 2 * pstData->astCoordList[2].dwY + pstData->astCoordList[1].dwY);

        // 计算预测坐标（包括加速度补偿）
        stPrediction.dwX = pstData->astCoordList[3].dwX + dx + ddx / 2; // 加速度补偿
        stPrediction.dwY = pstData->astCoordList[3].dwY + dy + ddy / 2; // 加速度补偿
        stPrediction.dwZ = 120;

        // 在X轴和Y轴上的位移
        gLct_dwSpeedX = (int32)dx + ddx / 2; // 位移速度
        gLct_dwSpeedY = (int32)dy + ddy / 2; // 位移速度

        // 保存预测坐标
        *pstPredictionCoord = stPrediction;

        // 保存上一个定位点坐标
        *pstAheadPointCoord = pstData->astCoordList[byMeanListLen - 1];

        // 预测成功
        *pbyPredictionFlag = 1;
    }
    // 不在同一大楼和楼层，系统进入基站捕获状态
    else
    {
        memset(&gLct_stOriginalLctCoordList, 0, sizeof(LCT_COORDSEL_ORIGINAL_COORD_T)); // 原始坐标队列复位
        memset(&gLct_stMeanLctCoordList, 0, sizeof(LCT_COORDSEL_MEAN_COORD_T));         // 平滑坐标队列复位
        gg_stSysStatus.byBtsCaptureFlag = 0;                                            // 系统进入基站捕获状态

        // 预测失败
        *pbyPredictionFlag = 0;
    }

    return;
}
